/*
Copyright (C) 2004 Douglas Thain
This work is made available under a BSD-style license.
See the file COPYING for details.
*/

#include "domain_name_cache.h"
#include "hash_cache.h"

#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/utsname.h>

/* cache domain names for up to five minutes */
#define DOMAIN_NAME_CACHE_LIFETIME 300

static struct hash_cache *name_to_addr=0;
static struct hash_cache *addr_to_name=0;

static int domain_name_cache_init()
{
	if(!name_to_addr) {
		name_to_addr = hash_cache_create(127,hash_string,free);
		if(!name_to_addr) {
			return 0;
		}
	}

	if(!addr_to_name) {
		addr_to_name = hash_cache_create(127,hash_string,free);
		if(!addr_to_name) {
			return 0;
		}
	}

	return 1;
}

int domain_name_cache_canonical( const char *name_or_addr, char *cname )
{
	char addr[DOMAIN_NAME_MAX];
	return domain_name_cache_lookup(name_or_addr,addr) &&
	domain_name_cache_lookup_reverse(addr,cname);
}

int domain_name_cache_lookup( const char *name, char *addr )
{
	char *found, *copy;
	int success;

	if(!domain_name_cache_init()) return 0;

	found = hash_cache_lookup(name_to_addr,name);
	if(found) {
		strcpy(addr,found);
		return 1;
	}

	success = domain_name_lookup(name,addr);
	if(!success) return 0;

	copy = strdup(addr);
	if(!copy) return 1;

	success = hash_cache_insert(name_to_addr,name,copy,DOMAIN_NAME_CACHE_LIFETIME);

	return 1;
}

int domain_name_cache_lookup_reverse( const char *addr, char *name )
{
	char *found, *copy;
	int success;

	if(!domain_name_cache_init()) return 0;

	found = hash_cache_lookup(addr_to_name,addr);
	if(found) {
		strcpy(name,found);
		return 1;
	}

	success = domain_name_lookup_reverse(addr,name);
	if(!success) return 0;

	copy = strdup(name);
	if(!copy) return 1;

	success = hash_cache_insert(addr_to_name,addr,copy,DOMAIN_NAME_CACHE_LIFETIME);

	return 1;
}

int domain_name_cache_guess( char *name )
{
	struct utsname n;
	char addr[DOMAIN_NAME_MAX];

	if(uname(&n)<0) return 0;
	if(!domain_name_cache_lookup(n.nodename,addr)) return 0;
	if(!domain_name_cache_lookup_reverse(addr,name)) return 0;

	return 1;
}

